@@ -1,7 +1,7 @@
 /*
  *  libzvbi - Closed Caption and Teletext rendering
  *
- *  Copyright (C) 2000, 2001, 2002, 2003, 2004 Michael H. Schimek
+ *  Copyright (C) 2000-2004 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: exp-gfx.c,v 1.50 2005/02/18 07:56:22 mschimek Exp $ */
+/* $Id: exp-gfx.c,v 1.53 2005/07/04 21:57:57 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -31,7 +31,7 @@
 #include "misc.h"
 #include "page.h"		/* vbi3_page */
 #include "lang.h"		/* vbi3_is_drcs() */
-#include "common/intl-priv.h"
+#include "intl-priv.h"
 #include "export-priv.h"	/* vbi3_export */
 #include "exp-gfx.h"
 #include "vt.h"			/* VBI3_TRANSPARENT_BLACK */
@@ -177,11 +177,11 @@ unicode_ccfont3			(unsigned int		c,
 	};
 	unsigned int i;
 
-	if (c < 0x0020)
+	if (c < 0x0020) {
 		c = 15; /* invalid */
-	else if (c < 0x0080)
+	} else if (c < 0x0080) {
 		c = c;
-	else {
+	} else {
 		for (i = 0; i < N_ELEMENTS (specials); ++i)
 			if (c == specials[i]) {
 				c = i + 6;
@@ -205,27 +205,42 @@ unicode_ccfont3			(unsigned int		c,
  */
 static void
 line_doubler			(void *			buffer,
-				 const vbi3_image_format *format)
+				 const vbi3_image_format *format,
+				 unsigned int		x,
+				 unsigned int		y,
+				 unsigned int		width,
+				 unsigned int		height)
 {
 	uint8_t *canvas;
 	unsigned int byte_width;
 	unsigned int bytes_per_line;
-	unsigned int height;
+	unsigned int bytes_per_pixel;
 
 	assert (VBI3_PIXFMT_IS_PACKED (format->pixfmt));
+	assert (x + width <= format->width);
+	assert (y + height <= format->height);
+	assert (0 == (height % 2));
 
-	canvas = ((uint8_t *) buffer) + format->offset;
+	bytes_per_pixel = vbi3_pixfmt_bytes_per_pixel (format->pixfmt);
+	byte_width = width * bytes_per_pixel;
 
-	byte_width = format->width
-		* vbi3_pixfmt_bytes_per_pixel (format->pixfmt);
 	bytes_per_line = format->bytes_per_line;
 
-	if (bytes_per_line <= 0)
+	if (bytes_per_line <= 0) {
 		bytes_per_line = byte_width;
+	} else {
+		assert (byte_width <= bytes_per_line);
+	}
 
-	for (height = format->height; height > 0; height -= 2) {
+	canvas = ((uint8_t *) buffer)
+		+ format->offset
+		+ y * bytes_per_line
+		+ x * bytes_per_pixel;
+
+	while (height > 0) {
 		memcpy (canvas + bytes_per_line, canvas, byte_width);
 		canvas += bytes_per_line * 2;
+		height -= 2;
 	}
 }
 
@@ -481,10 +496,9 @@ vbi3_rgba_conv			(void *			buffer,
 		break;
 
 	default:
-		vbi3_log_printf (VBI3_DEBUG, __FUNCTION__,
-				"Invalid pixfmt %u (%s)",
-				(unsigned int) pixfmt,
-				vbi3_pixfmt_name (pixfmt));
+		debug ("Invalid pixfmt %u (%s)",
+		       (unsigned int) pixfmt,
+		       vbi3_pixfmt_name (pixfmt));
 		return FALSE;
 	}
 
@@ -544,7 +558,7 @@ do {									\
 		s += (cpl) * (cw) / 8 * (ch) / 2;			\
 		under1 >>= (ch) / 2;					\
 									\
-	case VBI3_DOUBLE_HEIGHT:						\
+	case VBI3_DOUBLE_HEIGHT:					\
 	case VBI3_DOUBLE_SIZE: 						\
 		ch1 >>= 1;						\
 									\
@@ -569,7 +583,7 @@ do {									\
 			d += bpl;					\
 			break;						\
 									\
-		case VBI3_DOUBLE_HEIGHT:					\
+		case VBI3_DOUBLE_HEIGHT:				\
 		case VBI3_DOUBLE_HEIGHT2:				\
 			for (x = 0; x < (cw); bits >>= 1, ++x) {	\
 				PIXEL (d, x, pen, bits & 1);		\
@@ -583,7 +597,7 @@ do {									\
 				PIXEL (d, x, pen, bits & 1);		\
 				PIXEL (d, x + 1, pen, bits & 1);	\
 			}						\
-			d += bpl * 2;					\
+			d += bpl;					\
 			break;						\
 									\
 		case VBI3_DOUBLE_SIZE:					\
@@ -633,7 +647,7 @@ do {									\
 	case VBI3_DOUBLE_HEIGHT2:					\
 		s += 30;						\
 									\
-	case VBI3_DOUBLE_HEIGHT:						\
+	case VBI3_DOUBLE_HEIGHT:					\
 		for (y = 0; y < TCH / 2; d += bpl * 2, ++y)		\
 			for (x = 0; x < 12; ++s, x += 2) {		\
 				col = *s & 15;				\
@@ -719,15 +733,15 @@ do {									\
  * @param pg Source page.
  * @param buffer Image buffer.
  * @param format Pixel format and dimensions of the buffer.
- *   The buffer must be large enough for @a width x @a height characters
+ *   The buffer must be large enough for @a n_columns x @a height characters
  *   of 16 x 13 pixels each.
  * @param flags Optional set of the following flags:
  *   - @c VBI3_SCALE: Duplicate lines. In this case characters are 16 x 26
  *     pixels, suitable for frame (rather than field) overlay.
  * @param column First source column, 0 ... pg->columns - 1.
  * @param row First source row, 0 ... pg->rows - 1.
- * @param width Number of columns to draw, 1 ... pg->columns.
- * @param height Number of rows to draw, 1 ... pg->rows.
+ * @param n_columns Number of columns to draw, 1 ... pg->columns.
+ * @param n_rows Number of rows to draw, 1 ... pg->rows.
  * 
  * Draws a subsection of a Closed Caption vbi3_page.
  *
@@ -742,10 +756,12 @@ vbi3_page_draw_caption_region_va_list
 				(const vbi3_page *	pg,
 				 void *			buffer,
 				 const vbi3_image_format *format,
+				 unsigned int		x,
+				 unsigned int		y,
 				 unsigned int		column,
 				 unsigned int		row,
-				 unsigned int		width,
-				 unsigned int		height,
+				 unsigned int		n_columns,
+				 unsigned int		n_rows,
 				 va_list		export_options)
 {
 	vbi3_bool option_scale;
@@ -774,7 +790,8 @@ vbi3_page_draw_caption_region_va_list
 			ac = pg->text + i * pg->columns;
 
 			for (j = 0; j < pg->columns; ++j)
-				fprintf (stderr, "%d%d%02x ",
+				fprintf (stderr, "%d%d%d%02x ",
+					 ac[j].opacity,
 					 ac[j].foreground,
 					 ac[j].background,
 					 ac[j].unicode & 0xFF);
@@ -821,15 +838,33 @@ vbi3_page_draw_caption_region_va_list
 			break;
 	}
 
+	if (x >= format->width
+	    || y >= format->height) {
+		debug ("Position x %u, y %u is beyond image size %u x %u",
+		       x, y, format->width, format->height);
+		return FALSE;
+	}
+
+	if (column + n_columns > pg->columns
+	    || row + n_rows > pg->rows) {
+		debug ("Columns %u ... %u, rows %u ... %u beyond "
+		       "page size of %u x %u characters",
+		       column, column + n_columns - 1,
+		       row, row + n_rows - 1,
+		       pg->columns, pg->rows);
+		return FALSE;
+	}
+
 	scaled_height = option_scale ? 26 : 13;
 
-	if (width * 16 > format->width
-	    || height * scaled_height > format->height) {
-		vbi3_log_printf (VBI3_DEBUG, __FUNCTION__,
-				"Image size %u x %u too small for "
-				"%u x %u characters",
-				format->width, format->height,
-				width, height);
+	if (n_columns * 16 > format->width - x
+	    || n_rows * scaled_height > format->height - y) {
+		debug ("Image size %u x %u too small to draw %u x %u "
+		       "characters (%u x %u pixels) at x %u, y %u",
+		       format->width, format->height,
+		       n_columns, n_rows,
+		       n_columns * 16, n_rows * scaled_height,
+		       x, y);
 		return FALSE;
 	}
 
@@ -838,48 +873,50 @@ vbi3_page_draw_caption_region_va_list
 			    brightness, contrast))
 		return FALSE;
 
-	canvas = ((uint8_t *) buffer) + format->offset;
-
 	bytes_per_pixel = vbi3_pixfmt_bytes_per_pixel (format->pixfmt);
 	bytes_per_line = format->bytes_per_line;
 
 	if (bytes_per_line <= 0) {
 		bytes_per_line = pg->columns * CCW * bytes_per_pixel;
 	} else if ((format->width * bytes_per_pixel) > bytes_per_line) {
-		vbi3_log_printf (VBI3_DEBUG, __FUNCTION__,
-				"Image width %u (%s) > bytes_per_line %u",
-				format->width,
-				vbi3_pixfmt_name (format->pixfmt),
-				bytes_per_line);
+		debug ("Image width %u (%s) > bytes_per_line %u",
+		       format->width, vbi3_pixfmt_name (format->pixfmt),
+		       bytes_per_line);
 		return FALSE;
 	}
 
+	canvas = ((uint8_t *) buffer)
+		+ format->offset
+		+ y * bytes_per_line
+		+ x * bytes_per_pixel;
+
 	size = format->offset + bytes_per_line * format->height;
 
 	if (size > format->size) {
-		vbi3_log_printf (VBI3_DEBUG, __FUNCTION__,
-				"Image %u x %u, offset %u, bytes_per_line %u " 
-				"> buffer size %u = 0x%08x",
-				format->width, format->height,
-				format->offset, bytes_per_line,
-				format->size, format->size);
+		debug ("Image %u x %u, offset %u, bytes_per_line %u " 
+		       "> buffer size %u = 0x%08x",
+		       format->width, format->height,
+		       format->offset, bytes_per_line,
+		       format->size, format->size);
 		return FALSE;
 	}
 
 	if (option_scale)
 		bytes_per_line *= 2;
 
-	row_adv = bytes_per_line * CCH - bytes_per_pixel * width * CCW;
+	row_adv = bytes_per_line * CCH - bytes_per_pixel * n_columns * CCW;
 
 	switch (bytes_per_pixel) {
+		unsigned int rowct;
+
 	case 4:
-		for (; height-- > 0; ++row) {
+		for (rowct = n_rows; rowct-- > 0; ++row) {
 			const vbi3_char *ac;
-			unsigned int count;
+			unsigned int colct;
 
 			ac = pg->text + row * pg->columns + column;
 
-			for (count = width; count-- > 0; ++ac) {
+			for (colct = n_columns; colct-- > 0; ++ac) {
 				DRAW_CC_CHAR (4);
 				canvas += CCW * 4;
 			}
@@ -890,13 +927,13 @@ vbi3_page_draw_caption_region_va_list
 		break;
 
 	case 3:
-		for (; height-- > 0; ++row) {
+		for (rowct = n_rows; rowct-- > 0; ++row) {
 			const vbi3_char *ac;
-			unsigned int count;
+			unsigned int colct;
 
 			ac = pg->text + row * pg->columns + column;
 
-			for (count = width; count-- > 0; ++ac) {
+			for (colct = n_columns; colct-- > 0; ++ac) {
 				DRAW_CC_CHAR (3);
 				canvas += CCW * 3;
 			}
@@ -907,13 +944,13 @@ vbi3_page_draw_caption_region_va_list
 		break;
 
 	case 2:
-		for (; height-- > 0; ++row) {
+		for (rowct = n_rows; rowct-- > 0; ++row) {
 			const vbi3_char *ac;
-			unsigned int count;
+			unsigned int colct;
 
 			ac = pg->text + row * pg->columns + column;
 
-			for (count = width; count-- > 0; ++ac) {
+			for (colct = n_columns; colct-- > 0; ++ac) {
 				DRAW_CC_CHAR (2);
 				canvas += CCW * 2;
 			}
@@ -924,13 +961,13 @@ vbi3_page_draw_caption_region_va_list
 		break;
 
 	case 1:
-		for (; height-- > 0; ++row) {
+		for (rowct = n_rows; rowct-- > 0; ++row) {
 			const vbi3_char *ac;
-			unsigned int count;
+			unsigned int colct;
 
 			ac = pg->text + row * pg->columns + column;
 
-			for (count = width; count-- > 0; ++ac) {
+			for (colct = n_columns; colct-- > 0; ++ac) {
 				DRAW_CC_CHAR (1);
 				canvas += CCW;
 			}
@@ -945,7 +982,8 @@ vbi3_page_draw_caption_region_va_list
 	}
 
 	if (option_scale)
-		line_doubler (buffer, format);
+  		line_doubler (buffer, format, x, y,
+			      n_columns * 16, n_rows * scaled_height);
 
 	return TRUE;
 }
@@ -954,19 +992,23 @@ vbi3_bool
 vbi3_page_draw_caption_region	(const vbi3_page *	pg,
 				 void *			buffer,
 				 const vbi3_image_format *format,
+				 unsigned int		x,
+				 unsigned int		y,
 				 unsigned int		column,
 				 unsigned int		row,
-				 unsigned int		width,
-				 unsigned int		height,
+				 unsigned int		n_columns,
+				 unsigned int		n_rows,
 				 ...)
 {
 	vbi3_bool r;
 	va_list export_options;
 
-	va_start (export_options, height);
-	r = vbi3_page_draw_caption_region_va_list (pg, buffer, format,
-						  column, row, width, height,
-						  export_options);
+	va_start (export_options, n_rows);
+
+	r = vbi3_page_draw_caption_region_va_list
+		(pg, buffer, format, x, y, column, row, n_columns, n_rows,
+		 export_options);
+
 	va_end (export_options);
 
 	return r;
@@ -983,8 +1025,8 @@ vbi3_page_draw_caption_region	(const vbi3_page *	pg,
  *     pixels, suitable for frame (rather than field) overlay.
  * @param column First source column, 0 ... pg->columns - 1.
  * @param row First source row, 0 ... pg->rows - 1.
- * @param width Number of columns to draw, 1 ... pg->columns.
- * @param height Number of rows to draw, 1 ... pg->rows.
+ * @param n_columns Number of columns to draw, 1 ... pg->columns.
+ * @param n_rows Number of rows to draw, 1 ... pg->rows.
  * 
  * Draws a subsection of a Closed Caption vbi3_page.
  *
@@ -1000,11 +1042,12 @@ vbi3_page_draw_caption_va_list	(const vbi3_page *	pg,
 				 const vbi3_image_format *format,
 				 va_list		export_options)
 {
-	return vbi3_page_draw_caption_region_va_list (pg, buffer, format,
-						     /* column */ 0,
-						     /* row */ 0,
-						     pg->columns, pg->rows,
-						     export_options);
+	return vbi3_page_draw_caption_region_va_list
+		(pg, buffer, format,
+		 /* x */ 0, /* y */ 0,
+		 /* column */ 0, /* row */ 0,
+		 pg->columns, pg->rows,
+		 export_options);
 }
 
 vbi3_bool
@@ -1017,10 +1060,14 @@ vbi3_page_draw_caption		(const vbi3_page *	pg,
 	va_list export_options;
 
 	va_start (export_options, format);
-	r = vbi3_page_draw_caption_region_va_list (pg, buffer, format,
-						  /* column */ 0, /* row */ 0,
-						  pg->columns, pg->rows,
-						  export_options);
+
+	r = vbi3_page_draw_caption_region_va_list
+		(pg, buffer, format,
+		 /* x */ 0, /* y */ 0,
+		 /* column */ 0, /* row */ 0,
+		 pg->columns, pg->rows,
+		 export_options);
+
 	va_end (export_options);
 
 	return r;
@@ -1077,7 +1124,7 @@ do {									\
  * @param pg Source page.
  * @param buffer Image buffer.
  * @param format Pixel format and dimensions of the buffer. The buffer must be
- *   large enough for @a width x @a height characters of 12 x 10 pixels
+ *   large enough for @a n_columns x @a n_rows characters of 12 x 10 pixels
  *   each.
  * @param flags Optional set of the following flags:
  *   - @c VBI3_REVEAL: Draw characters flagged 'conceal' (see vbi3_char).
@@ -1087,8 +1134,8 @@ do {									\
  *     pixels, suitable for frame (rather than field) overlay.
  * @param column First source column, 0 ... pg->columns - 1.
  * @param row First source row, 0 ... pg->rows - 1.
- * @param width Number of columns to draw, 1 ... pg->columns.
- * @param height Number of rows to draw, 1 ... pg->rows.
+ * @param n_columns Number of columns to draw, 1 ... pg->columns.
+ * @param n_rows Number of rows to draw, 1 ... pg->rows.
  * 
  * Draws a subsection of a Teletext vbi3_page.
  *
@@ -1103,10 +1150,12 @@ vbi3_page_draw_teletext_region_va_list
 				(const vbi3_page *	pg,
 				 void *			buffer,
 				 const vbi3_image_format *format,
+				 unsigned int		x,
+				 unsigned int		y,
 				 unsigned int		column,
 				 unsigned int		row,
-				 unsigned int		width,
-				 unsigned int		height,
+				 unsigned int		n_columns,
+				 unsigned int		n_rows,
 				 va_list		export_options)
 {
 	vbi3_bool option_scale;
@@ -1117,8 +1166,8 @@ vbi3_page_draw_teletext_region_va_list
 	uint8_t *canvas;
 	unsigned int scaled_height;
 	unsigned int bytes_per_pixel;
-	unsigned int bytes_per_line;
-	unsigned int size;
+	unsigned long bytes_per_line;
+	unsigned long size;
 	unsigned int row_adv;
 
 	assert (NULL != pg);
@@ -1189,15 +1238,33 @@ vbi3_page_draw_teletext_region_va_list
 			break;
 	}
 
+	if (x >= format->width
+	    || y >= format->height) {
+		debug ("Position x %u, y %u is beyond image size %u x %u",
+		       x, y, format->width, format->height);
+		return FALSE;
+	}
+
+	if (column + n_columns > pg->columns
+	    || row + n_rows > pg->rows) {
+		debug ("Columns %u ... %u, rows %u ... %u beyond "
+		       "page size of %u x %u characters",
+		       column, column + n_columns - 1,
+		       row, row + n_rows - 1,
+		       pg->columns, pg->rows);
+		return FALSE;
+	}
+
 	scaled_height = option_scale ? 20 : 10;
 
-	if (width * 12 > format->width
-	    || height * scaled_height > format->height) {
-		vbi3_log_printf (VBI3_DEBUG, __FUNCTION__,
-				"Image size %u x %u too small for "
-				"%u x %u characters",
-				format->width, format->height,
-				width, height);
+	if (n_columns * 12 > format->width - x
+	    || n_rows * scaled_height > format->height - x) {
+		debug ("Image size %u x %u too small to draw %u x %u "
+		       "characters (%u x %u pixels) at x %u, y %u",
+		       format->width, format->height,
+		       n_columns, n_rows,
+		       n_columns * 12, n_rows * scaled_height,
+		       x, y);
 		return FALSE;
 	}
 
@@ -1206,57 +1273,58 @@ vbi3_page_draw_teletext_region_va_list
 			    brightness, contrast))
 		return FALSE;
 
-	canvas = ((uint8_t *) buffer) + format->offset;
-
 	bytes_per_pixel = vbi3_pixfmt_bytes_per_pixel (format->pixfmt);
 	bytes_per_line = format->bytes_per_line;
 
 	if (bytes_per_line <= 0) {
 		bytes_per_line = pg->columns * TCW * bytes_per_pixel;
 	} else if ((format->width * bytes_per_pixel) > bytes_per_line) {
-		vbi3_log_printf (VBI3_DEBUG, __FUNCTION__,
-				"Image width %u (%s) > bytes_per_line %u",
-				format->width,
-				vbi3_pixfmt_name (format->pixfmt),
-				bytes_per_line);
+		debug ("Image width %u (%s) > bytes_per_line %lu",
+		       format->width, vbi3_pixfmt_name (format->pixfmt),
+		       bytes_per_line);
 		return FALSE;
 	}
 
+	canvas = ((uint8_t *) buffer)
+		+ format->offset
+		+ y * bytes_per_line
+		+ x * bytes_per_pixel;
+
 	size = format->offset + bytes_per_line * format->height;
 
 	if (size > format->size) {
-		vbi3_log_printf (VBI3_DEBUG, __FUNCTION__,
-				"Image %u x %u, offset %u, bytes_per_line %u "
-				"> buffer size %u = 0x%08x",
-				format->width, format->height,
-				format->offset, bytes_per_line,
-				format->size, format->size);
+		debug ("Image %u x %u, offset %lu, bytes_per_line %lu "
+		       "> buffer size %lu = 0x%08lx",
+		       format->width, format->height,
+		       format->offset, bytes_per_line,
+		       format->size, format->size);
 		return FALSE;
 	}
 
 	if (option_scale)
 		bytes_per_line *= 2;
 
-	row_adv = bytes_per_line * TCH - bytes_per_pixel * width * TCW;
+	row_adv = bytes_per_line * TCH - bytes_per_pixel * n_columns * TCW;
 
 	switch (bytes_per_pixel) {
 	case 4:
 	{
 		uint32_t pen [2 + 8 + 32];
 		unsigned int i;
+		unsigned int rowct;
 
 		if (pg->drcs_clut)
 			for (i = 2; i < 2 + 8 + 32; i++)
 				pen[i] = ((uint32_t *) color_map)
 					[pg->drcs_clut[i]];
 
-		for (; height-- > 0; ++row) {
+		for (rowct = n_rows; rowct-- > 0; ++row) {
 			const vbi3_char *ac;
-			unsigned int count;
+			unsigned int colct;
 
 			ac = pg->text + row * pg->columns + column;
 
-			for (count = width; count-- > 0; ++ac) {
+			for (colct = n_columns; colct-- > 0; ++ac) {
 				DRAW_VT_CHAR (4);
 				canvas += TCW * 4;
 			}
@@ -1272,18 +1340,19 @@ vbi3_page_draw_teletext_region_va_list
 		const unsigned int bpp = 3;
 		uint8_t pen [(2 + 8 + 32) * 3];
 		unsigned int i;
+		unsigned int rowct;
 
 		if (pg->drcs_clut)
 			for (i = 2; i < 2 + 8 + 32; i++)
 				PIXEL (pen, i, color_map, pg->drcs_clut[i]);
 
-		for (; height-- > 0; ++row) {
+		for (rowct = n_rows; rowct-- > 0; ++row) {
 			const vbi3_char *ac;
-			unsigned int count;
+			unsigned int colct;
 
 			ac = pg->text + row * pg->columns + column;
 
-			for (count = width; count-- > 0; ++ac) {
+			for (colct = n_columns; colct-- > 0; ++ac) {
 				DRAW_VT_CHAR (3);
 				canvas += TCW * 3;
 			}
@@ -1298,19 +1367,20 @@ vbi3_page_draw_teletext_region_va_list
 	{
 		uint16_t pen [2 + 8 + 32];
 		unsigned int i;
+		unsigned int rowct;
 
 		if (pg->drcs_clut)
 			for (i = 2; i < 2 + 8 + 32; i++)
 				pen[i] = ((uint16_t *) color_map)
 					[pg->drcs_clut[i]];
 
-		for (; height-- > 0; ++row) {
+		for (rowct = n_rows; rowct-- > 0; ++row) {
 			const vbi3_char *ac;
-			unsigned int count;
+			unsigned int colct;
 
 			ac = pg->text + row * pg->columns + column;
 
-			for (count = width; count-- > 0; ++ac) {
+			for (colct = n_columns; colct-- > 0; ++ac) {
 				DRAW_VT_CHAR (2);
 				canvas += TCW * 2;
 			}
@@ -1325,19 +1395,20 @@ vbi3_page_draw_teletext_region_va_list
 	{
 		uint8_t pen [2 + 8 + 32];
 		unsigned int i;
+		unsigned int rowct;
 
 		if (pg->drcs_clut)
 			for (i = 2; i < 2 + 8 + 32; i++)
 				pen[i] = ((uint8_t *) color_map)
 					[pg->drcs_clut[i]];
 
-		for (; height-- > 0; ++row) {
+		for (rowct = n_rows; rowct-- > 0; ++row) {
 			const vbi3_char *ac;
-			unsigned int count;
+			unsigned int colct;
 
 			ac = pg->text + row * pg->columns + column;
 
-			for (count = width; count-- > 0; ++ac) {
+			for (colct = n_columns; colct-- > 0; ++ac) {
 				DRAW_VT_CHAR (1);
 				canvas += TCW;
 			}
@@ -1353,7 +1424,8 @@ vbi3_page_draw_teletext_region_va_list
 	}
 
 	if (option_scale)
-		line_doubler (buffer, format);
+		line_doubler (buffer, format, x, y,
+			      n_columns * 12, n_rows * scaled_height);
 
 	return TRUE;
 }
@@ -1362,19 +1434,24 @@ vbi3_bool
 vbi3_page_draw_teletext_region	(const vbi3_page *	pg,
 				 void *			buffer,
 				 const vbi3_image_format *format,
+				 unsigned int		x,
+				 unsigned int		y,
 				 unsigned int		column,
 				 unsigned int		row,
-				 unsigned int		width,
-				 unsigned int		height,
+				 unsigned int		n_columns,
+				 unsigned int		n_rows,
 				 ...)
 {
 	vbi3_bool r;
 	va_list export_options;
 
-	va_start (export_options, height);
-	r = vbi3_page_draw_teletext_region_va_list (pg, buffer, format,
-						   column, row, width, height,
-						   export_options);
+	va_start (export_options, n_rows);
+
+	r = vbi3_page_draw_teletext_region_va_list
+		(pg, buffer, format, x, y,
+		 column, row, n_columns, n_rows,
+		 export_options);
+
 	va_end (export_options);
 
 	return r;
@@ -1407,11 +1484,12 @@ vbi3_page_draw_teletext_va_list	(const vbi3_page *	pg,
 				 const vbi3_image_format *format,
 				 va_list		export_options)
 {
-	return vbi3_page_draw_teletext_region_va_list (pg, buffer, format,
-						      /* column */ 0,
-						      /* row */ 0,
-						      pg->columns, pg->rows,
-						      export_options);
+	return vbi3_page_draw_teletext_region_va_list
+		(pg, buffer, format,
+		 /* x */ 0, /* y */ 0,
+		 /* column */ 0, /* row */ 0,
+		 pg->columns, pg->rows,
+		 export_options);
 }
 
 vbi3_bool
@@ -1424,10 +1502,14 @@ vbi3_page_draw_teletext		(const vbi3_page *	pg,
 	va_list export_options;
 
 	va_start (export_options, format);
-	r = vbi3_page_draw_teletext_region_va_list (pg, buffer, format,
-						   /* column */ 0, /* row */ 0,
-						   pg->columns, pg->rows,
-						   export_options);
+
+	r = vbi3_page_draw_teletext_region_va_list
+		(pg, buffer, format,
+		 /* x */ 0, /* y */ 0,
+		 /* column */ 0, /* row */ 0,
+		 pg->columns, pg->rows,
+		 export_options);
+
 	va_end (export_options);
 
 	return r;
@@ -1567,13 +1649,15 @@ export_ppm			(vbi3_export *		e,
 		if (pg->columns < 40)
 			success = vbi3_page_draw_caption_region
 				(pg, image, &format,
+				 /* x */ 0, /* y */ 0,
 				 /* column */ 0, /* row */ row,
 				 pg->columns, /* rows */ 1,
-				 /* options */ 0);
+				 /* options */ VBI3_END);
 		else
 #endif
 			success = vbi3_page_draw_teletext_region
 				(pg, image, &format,
+				 /* x */ 0, /* y */ 0,
 				 /* column */ 0, /* row */ row,
 				 pg->columns, /* rows */ 1,
 				 /* options: */
@@ -1653,6 +1737,11 @@ _vbi3_export_module_ppm = {
 #include "png.h"
 #include "setjmp.h"
 
+#define COLOR_MAP_SIZE (N_ELEMENTS (pg->color_map))
+#define TRANSLUCENT COLOR_MAP_SIZE
+#define TRANSPARENT (COLOR_MAP_SIZE * 2)
+#define PALETTE_SIZE (TRANSPARENT + 1)
+
 static void
 png_draw_char			(uint8_t *		canvas,
 				 unsigned int		bytes_per_line,
@@ -1672,7 +1761,7 @@ png_draw_char			(uint8_t *		canvas,
 		/* Transparent foreground and background. */
 
 	blank:
-		pen[0] = VBI3_TRANSPARENT_BLACK;
+		pen[0] = TRANSPARENT;
 
 		if (is_ttx)
 			DRAW_BLANK (canvas, 1, bytes_per_line, pen, TCW, TCH);
@@ -1683,26 +1772,24 @@ png_draw_char			(uint8_t *		canvas,
 		return;
 
 	case VBI3_TRANSPARENT_FULL:
-		/* Transparent background, opaque foreground. Currently not
-		   used. Mind Teletext level 2.5 foreground and background
-		   transparency by referencing colormap entry 8,
-		   VBI3_TRANSPARENT_BLACK. The background of multicolor DRCS is
+		/* Transparent background, opaque foreground.
+		   The background of Teletext multicolor DRCS is
 		   ambiguous, so we make them opaque. */
 
-		pen[0] = VBI3_TRANSPARENT_BLACK;
+		pen[0] = TRANSPARENT;
 		pen[1] = ac->foreground;
 
 		break;
 
-	case VBI3_SEMI_TRANSPARENT:
+	case VBI3_TRANSLUCENT:
 		/* Translucent background (for 'boxed' text), opaque
-		   foreground. The background of multicolor DRCS is ambiguous,
-		   so we make them completely translucent. */
+		   foreground. The background of Teletext multicolor DRCS
+		   is ambiguous, so we make them completely translucent. */
 
 		if (vbi3_is_drcs (unicode))
 			pen += 64; /* use translucent DRCS palette */
 
-		pen[0] = ac->background + 40; /* translucent */
+		pen[0] = ac->background + TRANSLUCENT;
 		pen[1] = ac->foreground;
 
 		break;
@@ -1751,6 +1838,118 @@ png_draw_char			(uint8_t *		canvas,
 	}
 }
 
+static vbi3_bool
+write_png			(vbi3_export *		e,
+				 const vbi3_page *	pg,
+				 png_structp		png_ptr,
+				 png_infop		info_ptr,
+				 png_bytep		image,
+				 png_bytep *		row_pointer,
+				 const vbi3_image_format *format,
+				 vbi3_bool		double_height)
+{
+	png_color palette[PALETTE_SIZE];
+	png_byte alpha[PALETTE_SIZE];
+	png_text text[4];
+	char title[80];
+	unsigned int i;
+	unsigned int size;
+
+	if (setjmp (png_ptr->jmpbuf))
+		return FALSE;
+
+	png_init_io (png_ptr, e->fp);
+
+	png_set_IHDR (png_ptr, info_ptr,
+		      format->width,
+		      format->height << double_height,
+		      /* bit_depth */ 8,
+		      PNG_COLOR_TYPE_PALETTE,
+		      double_height ? PNG_INTERLACE_ADAM7 : PNG_INTERLACE_NONE,
+		      PNG_COMPRESSION_TYPE_DEFAULT,
+		      PNG_FILTER_TYPE_DEFAULT);
+
+	/* Could be optimized (or does libpng?) */
+	for (i = 0; i < COLOR_MAP_SIZE; ++i) {
+		/* Opaque. */
+		palette[i].red   = VBI3_R (pg->color_map[i]);
+		palette[i].green = VBI3_G (pg->color_map[i]);
+		palette[i].blue	 = VBI3_B (pg->color_map[i]);
+		alpha[i]	 = 255;
+
+		/* Translucent. */
+		palette[i + TRANSLUCENT] = palette[i];
+		alpha[i + TRANSLUCENT] = 128;
+	}
+
+	/* Transparent (black) background. */
+	palette[TRANSPARENT].red   = 0;
+	palette[TRANSPARENT].green = 0;
+	palette[TRANSPARENT].blue  = 0;
+	alpha[TRANSPARENT] = 0;
+
+	png_set_PLTE (png_ptr, info_ptr, palette, 80);
+	png_set_tRNS (png_ptr, info_ptr, alpha, 80, NULL);
+
+	png_set_gAMA (png_ptr, info_ptr, 1.0 / 2.2);
+
+	size = 0;
+
+	if (e->network)
+		size = snprintf (title, sizeof (title),
+				 "%s ", e->network);
+	else
+		title[0] = 0;
+
+	/* FIXME
+	   ISO 8859-1 (Latin-1) character set required,
+	   see png spec for other */
+	if (pg->pgno < 0x100) {
+		size += snprintf (title + size, sizeof (title) - size,
+				  "Closed Caption"); /* proper name */
+	} else if (VBI3_ANY_SUBNO != pg->subno) {
+		size += snprintf (title + size, sizeof (title) - size,
+				  _("Teletext Page %3x.%x"),
+				  pg->pgno, pg->subno);
+	} else {
+		size += snprintf (title + size, sizeof (title) - size,
+				  _("Teletext Page %3x"), pg->pgno);
+	}
+
+	CLEAR (text);
+
+	text[0].key = "Title";
+	text[0].text = title;
+	text[0].compression = PNG_TEXT_COMPRESSION_NONE;
+
+	text[1].key = "Software";
+	text[1].text = e->creator;
+	text[1].compression = PNG_TEXT_COMPRESSION_NONE;
+
+	png_set_text (png_ptr, info_ptr, text, 2);
+
+	png_write_info (png_ptr, info_ptr);
+
+	if (double_height) {
+		for (i = 0; i < format->height; ++i) {
+			uint8_t *s;
+
+			s = image + format->bytes_per_line * i;
+			row_pointer[i * 2 + 0] = s;
+			row_pointer[i * 2 + 1] = s;
+		}
+	} else {
+		for (i = 0; i < format->height; ++i)
+			row_pointer[i] = image + format->bytes_per_line * i;
+	}
+
+	png_write_image (png_ptr, row_pointer);
+
+	png_write_end (png_ptr, info_ptr);
+
+	return TRUE;
+}
+
 static vbi3_bool
 export_png			(vbi3_export *		e,
 				 const vbi3_page *	pg)
@@ -1758,19 +1957,14 @@ export_png			(vbi3_export *		e,
 	gfx_instance *gfx = PARENT (e, gfx_instance, export);
 	vbi3_image_format format;
 	unsigned int cw, ch;
-	png_bytep *row_pointer;
 	png_bytep image;
+	png_bytep *row_pointer;
 	unsigned int row_adv;
 	png_byte pen[128];
 	png_bytep canvas;
 	unsigned int row;
-	unsigned int i;
 	png_structp png_ptr;
 	png_infop info_ptr;
-	png_color palette[80];
-	png_byte alpha[80];
-	png_text text[4];
-	char title[80];
 
 	if (pg->columns < 40) { /* caption */
 #ifdef ZAPPING8
@@ -1801,17 +1995,20 @@ export_png			(vbi3_export *		e,
 
 	if (NULL == image) {
 		_vbi3_export_malloc_error (e);
-		vbi3_free(row_pointer);
+		vbi3_free (row_pointer);
 		return FALSE;
 	}
 
 	row_adv = pg->columns * cw * (ch - 1);
 
-	if (pg->drcs_clut)
+	if (pg->drcs_clut) {
+		unsigned int i;
+
 		for (i = 2; i < 2 + 8 + 32; ++i) {
 			pen[i]      = pg->drcs_clut[i]; /* opaque */
-			pen[i + 64] = pg->drcs_clut[i] + 40; /* translucent */
+			pen[i + 64] = pg->drcs_clut[i] + TRANSLUCENT;
 		}
+	}
 
 	canvas = image;
 
@@ -1842,7 +2039,7 @@ export_png			(vbi3_export *		e,
 				       ac,
 				       (unsigned int) !e->reveal,
 				       pen,
-				       pg->columns >= 40);
+				       /* is_ttx */ pg->columns >= 40);
 
 			switch (ac->size) {
 			case VBI3_DOUBLE_WIDTH:
@@ -1874,111 +2071,15 @@ export_png			(vbi3_export *		e,
 		goto unknown_error;
 	}
 
-	/* Avoid possible longjmp breakage due to libpng ugliness */
-	/* XXX nested functions not portable. */
-	{ int do_write() {
-
-	if (setjmp (png_ptr->jmpbuf))
-		return 1;
-
-	png_init_io (png_ptr, e->fp);
-
-	png_set_IHDR (png_ptr, info_ptr,
-		      format.width,
-		      format.height << gfx->double_height,
-		      /* bit_depth */ 8,
-		      PNG_COLOR_TYPE_PALETTE,
-		      (gfx->double_height) ?
-		              PNG_INTERLACE_ADAM7 : PNG_INTERLACE_NONE,
-		      PNG_COMPRESSION_TYPE_DEFAULT,
-		      PNG_FILTER_TYPE_DEFAULT);
-
-	/* Could be optimized (or does libpng?) */
-	for (i = 0; i < 40; ++i) {
-		/* opaque */
-		palette[i].red   = pg->color_map[i] & 0xFF;
-		palette[i].green = (pg->color_map[i] >> 8) & 0xFF;
-		palette[i].blue	 = (pg->color_map[i] >> 16) & 0xFF;
-		alpha[i]	 = 255;
-
-		/* translucent */
-		palette[i + 40]  = palette[i];
-		alpha[i + 40]	 = 128;
+	if (!write_png (e, pg, png_ptr, info_ptr,
+			image, row_pointer, &format,
+			gfx->double_height)) {
+		png_destroy_write_struct (&png_ptr, &info_ptr);
+		goto write_error;
 	}
 
-	alpha[VBI3_TRANSPARENT_BLACK] = 0;
-	alpha[40 + VBI3_TRANSPARENT_BLACK] = 0;
-
-	png_set_PLTE (png_ptr, info_ptr, palette, 80);
-	png_set_tRNS (png_ptr, info_ptr, alpha, 80, NULL);
-
-	png_set_gAMA (png_ptr, info_ptr, 1.0 / 2.2);
-
-	{
-		unsigned int size = 0;
-
-		if (e->network)
-			size = snprintf (title, sizeof (title),
-					 "%s ", e->network);
-		else
-			title[0] = 0;
-
-		/*
-		 *  FIXME
-		 *  ISO 8859-1 (Latin-1) character set required,
-		 *  see png spec for other
-		 */
-		if (pg->pgno < 0x100) {
-			size += snprintf (title + size, sizeof (title) - size,
-					  "Closed Caption"); /* proper name */
-		} else if (VBI3_ANY_SUBNO != pg->subno) {
-			size += snprintf (title + size, sizeof(title) - size,
-					  _("Teletext Page %3x.%x"),
-					  pg->pgno, pg->subno);
-		} else {
-			size += snprintf (title + size, sizeof(title) - size,
-					  _("Teletext Page %3x"), pg->pgno);
-		}
-	}
-
-	CLEAR (text);
-
-	text[0].key = "Title";
-	text[0].text = title;
-	text[0].compression = PNG_TEXT_COMPRESSION_NONE;
-
-	text[1].key = "Software";
-	text[1].text = e->creator;
-	text[1].compression = PNG_TEXT_COMPRESSION_NONE;
-
-	png_set_text (png_ptr, info_ptr, text, 2);
-
-	png_write_info (png_ptr, info_ptr);
-
-	if (gfx->double_height) {
-		for (i = 0; i < format.height; ++i) {
-			uint8_t *s;
-
-			s = image + format.bytes_per_line * i;
-			row_pointer[i * 2 + 0] = s;
-			row_pointer[i * 2 + 1] = s;
-		}
-	} else {
-		for (i = 0; i < format.height; ++i)
-			row_pointer[i] = image + format.bytes_per_line * i;
-	}
-
-	png_write_image (png_ptr, row_pointer);
-
-	png_write_end (png_ptr, info_ptr);
-
 	png_destroy_write_struct (&png_ptr, &info_ptr);
 
-	return 0;
-
-	/* See setjmp above */
-	} if (do_write ()) goto write_error; }
-
 	vbi3_free (row_pointer);
 	vbi3_free (image);
 
