@@ -19,7 +19,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: mpeg.c,v 1.57 2005/02/28 22:34:56 mschimek Exp $ */
+/* $Id: mpeg.c,v 1.59 2005/04/21 04:48:51 mschimek Exp $ */
 
 /* XXX gtk+ 2.3 GtkOptionMenu -> ? */
 #undef GTK_DISABLE_DEPRECATED
@@ -148,7 +148,6 @@ video_callback			(rte_context *		context _unused_,
       --stopped;
 
     b = zf_wait_full_buffer (&mpeg_consumer);
-
     cf = PARENT (b, capture_frame, b);
 
     /* XXX copy? */
@@ -173,7 +172,17 @@ video_unref			(rte_context *		context _unused_,
 				 rte_codec *		codec _unused_,
 				 rte_buffer *		rb)
 {
-  zf_send_empty_buffer (&mpeg_consumer, (zf_buffer *) rb->user_data);
+  zf_buffer *b;
+
+  if (0 == rb->size) {
+    /* Bug in librte < 0.5.6: If video_callback() returned FALSE
+       it passes an "EOF" buffer to the codec and later thinks
+       it must return data to this function. */
+    return TRUE;
+  }
+
+  b = (zf_buffer *) rb->user_data;
+  zf_send_empty_buffer (&mpeg_consumer, b);
   return TRUE;
 }
 
@@ -1934,6 +1943,8 @@ plugin_load_config		(gchar *		root_key)
   gchar *default_filename;
   guint n = strlen (root_key);
 
+  D();
+
   g_assert (n > 0 && root_key[n - 1] == '/');
   zconf_root = g_strndup (root_key, n - 1);
   zconf_root_temp = g_strconcat (zconf_root, "/temp", NULL);
@@ -1943,6 +1954,8 @@ plugin_load_config		(gchar *		root_key)
   zconf_get_string (&record_config_name, buffer);
   g_free (buffer);
 
+  D();
+
   default_filename = g_strconcat (getenv ("HOME"), "/clips/clip1", NULL);
   LOAD_CONFIG (string, default_filename, filename, "Last filename");
   g_free (default_filename);
@@ -2152,10 +2165,14 @@ plugin_init			(PluginBridge		bridge _unused_,
     add: properties_add
   };
 
+  D();
+
   zapping_info = info;
 
   append_property_handler (&mpeg_handler);
 
+  D();
+
   cmd_register ("record", py_record, METH_VARARGS,
 		("Record dialog"), "zapping.record()");
   cmd_register ("quickrec", py_quickrec, METH_VARARGS,
