@@ -19,7 +19,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: view.c,v 1.13 2005/01/31 07:06:47 mschimek Exp $ */
+/* $Id: view.c,v 1.16 2005/07/04 21:55:07 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -975,7 +975,7 @@ get_page			(const vbi3_network *	nk,
     {
       pg = vbi3_teletext_decoder_get_page
 	(td, nk, pgno, subno,
-	 VBI3_41_COLUMNS, TRUE, /* add_column, */
+	 VBI3_PADDING, TRUE, /* add_column, */
 	 /* VBI3_PANELS, FALSE, */
 	 VBI3_NAVIGATION, navigation,
 	 VBI3_HYPERLINKS, hyperlinks,
@@ -988,7 +988,7 @@ get_page			(const vbi3_network *	nk,
     {
       pg = vbi3_teletext_decoder_get_page
 	(td, nk, pgno, subno,
-	 VBI3_41_COLUMNS, TRUE, /* add_column, */
+	 VBI3_PADDING, TRUE, /* add_column, */
 	 /* VBI3_PANELS, FALSE, */
 	 VBI3_NAVIGATION, navigation,
 	 VBI3_HYPERLINKS, hyperlinks,
@@ -1076,7 +1076,7 @@ update_header			(TeletextView *		view,
 	 ev->network,
 	 ev->ev.ttx_page.pgno,
 	 ev->ev.ttx_page.subno,
-	 VBI3_41_COLUMNS, TRUE,
+	 VBI3_PADDING, TRUE,
 	 VBI3_HEADER_ONLY, TRUE,
 	 VBI3_WST_LEVEL, VBI3_WST_LEVEL_1p5,
 	 VBI3_OVERRIDE_CHARSET_0, view->charset,
@@ -1087,7 +1087,7 @@ update_header			(TeletextView *		view,
 	 ev->network,
 	 ev->ev.ttx_page.pgno,
 	 ev->ev.ttx_page.subno,
-	 VBI3_41_COLUMNS, TRUE,
+	 VBI3_PADDING, TRUE,
 	 VBI3_HEADER_ONLY, TRUE,
 	 VBI3_WST_LEVEL, VBI3_WST_LEVEL_1p5,
 	 VBI3_DEFAULT_CHARSET_0, default_charset,
@@ -1129,17 +1129,19 @@ update_header			(TeletextView *		view,
   buffer += column * (CW * 4);
 
   success = vbi3_page_draw_teletext_region (pg,
-					   buffer,
-					   &format,
-					   column,
-					   /* row */ 0,
-					   /* width */ 40 - column,
-					   /* height */ 1,
-					   VBI3_BRIGHTNESS, brightness,
-					   VBI3_CONTRAST, contrast,
-					   VBI3_REVEAL, TRUE,
-					   VBI3_FLASH_ON, TRUE,
-					   VBI3_END);
+					    buffer,
+					    &format,
+					    /* x */ 0,
+					    /* y */ 0,
+					    column,
+					    /* row */ 0,
+					    /* width */ 40 - column,
+					    /* height */ 1,
+					    VBI3_BRIGHTNESS, brightness,
+					    VBI3_CONTRAST, contrast,
+					    VBI3_REVEAL, TRUE,
+					    VBI3_FLASH_ON, TRUE,
+					    VBI3_END);
   g_assert (success);
 
   add_patch (view,
@@ -1931,6 +1933,8 @@ selection_get			(GtkWidget *		widget,
 	      (view->select.pg,
 	       gdk_pixbuf_get_pixels (pixbuf),
 	       &format,
+	       /* x */ 0,
+	       /* y */ 0,
 	       view->select.column1,
 	       view->select.row1,
 	       width,
@@ -2395,11 +2399,12 @@ select_start			(TeletextView *		view,
 	Export
 */
 
+extern gchar *zvbi_get_current_network_name (void); /* XXX */
+
 static void
 export_action			(GtkAction *		action _unused_,
 				 TeletextView *		view)
 {
-  extern gchar *zvbi_get_current_network_name (void); /* XXX */
   GtkWidget *dialog;
   gchar *name;
 
