@@ -65,10 +65,6 @@ Zapping *		zapping;
 
 /*** END OF GLOBAL STUFF ***/
 
-static gboolean		disable_vbi = FALSE; /* TRUE for disabling VBI
-						support */
-
-
 void shutdown_zapping(void);
 static gboolean startup_zapping(gboolean load_plugins,
 				tveng_device_info *info);
@@ -95,9 +91,6 @@ startup_teletext(void)
 #ifdef HAVE_LIBZVBI
   if (_teletext_view_new /* have Teletext plugin */)
     {
-      if (disable_vbi)
-	zconf_set_boolean(FALSE, "/zapping/options/vbi/enable_vbi");
-
       /* Make the vbi module open the device */
       D();
       zconf_touch("/zapping/options/vbi/enable_vbi");
@@ -106,7 +99,7 @@ startup_teletext(void)
   else
 #endif
     {
-      zconf_set_boolean(FALSE, "/zapping/options/vbi/enable_vbi");
+      disable_vbi = TRUE;
       vbi_gui_sensitive(FALSE);
     }
 }
@@ -191,7 +184,6 @@ restore_controls		(void)
   tveng_tc_control *controls;
   guint num_controls;
   gboolean start_muted;
-  tveng_tuned_channel *ch;
 
   D();
 
@@ -221,37 +213,7 @@ restore_controls		(void)
   /* XXX make this optional */
 
   if (1)
-    {
-      if (zcg_int(NULL, "current_input"))
-	z_switch_video_input(zcg_uint(NULL, "current_input"), zapping->info);
-
-      if (zcg_int(NULL, "current_audio_input"))
-	z_switch_audio_input(zcg_uint(NULL, "current_audio_input"),
-			     zapping->info);
-
-      if (zcg_int(NULL, "current_standard"))
-	z_switch_standard(zcg_uint(NULL, "current_standard"), zapping->info);
-
-      cur_tuned_channel = zcg_int(NULL, "cur_tuned_channel");
-      ch = tveng_tuned_channel_nth (global_channel_list,
-				    (guint) cur_tuned_channel);
-
-      if (NULL != ch)
-	{
-	  if (start_muted)
-	    {
-	      tveng_tc_control *mute;
-
-	      if ((mute = tveng_tc_control_by_id (zapping->info,
-						  ch->controls,
-						  ch->num_controls,
-						  TV_CONTROL_ID_MUTE)))
-		mute->value = 1; /* XXX sub-optimal */
-	    }
-
-	  z_switch_channel (ch, zapping->info);
-	}
-    }
+    zconf_get_sources (zapping->info, start_muted);
 }
 
 static void
@@ -323,6 +285,7 @@ int main(int argc, char * argv[])
   char *command = NULL;
   char *yuv_format = NULL;
   char *norm = NULL;
+  char *cpu_feature_str = NULL;
   gboolean mutable = TRUE;
   const gchar *display_name;
   tveng_device_info *info;
@@ -520,6 +483,7 @@ int main(int argc, char * argv[])
       0,
       /* TRANSLATORS: --yuv-format command line option. */
       N_("Obsolete"),
+      NULL
     },
     {
       "tunerless-norm",
@@ -529,9 +493,25 @@ int main(int argc, char * argv[])
       0,
       /* TRANSLATORS: --tunerless-norm command line option. */
       N_("Obsolete"),
+      NULL
+    },
+    {
+      "cpu-features",
+      0,
+      POPT_ARG_STRING,
+      &cpu_feature_str,
+      0,
+      N_("Override CPU detection"),
+      NULL
     },
     {
       NULL,
+      0,
+      0,
+      NULL,
+      0,
+      NULL,
+      NULL
     } /* end the list */
   };
 
@@ -576,11 +556,25 @@ int main(int argc, char * argv[])
     }
 
   printv("%s\n%s %s, build date: %s\n",
-	 "$Id: main.c,v 1.198 2005/02/25 18:13:50 mschimek Exp $",
+	 "$Id: main.c,v 1.203 2005/06/28 19:14:15 mschimek Exp $",
 	 "Zapping", VERSION, __DATE__);
 
   cpu_detection ();
-  printv ("CPU features 0x%x\n", cpu_features);
+
+  if (cpu_feature_str)
+    {
+      cpu_feature_set actual_features;
+
+      actual_features = cpu_features;
+      cpu_features &= cpu_feature_set_from_string (cpu_feature_str);
+
+      printv ("CPU features 0x%x (actual 0x%x)\n",
+    	      cpu_features, actual_features);
+    }
+  else
+    {
+      printv ("CPU features 0x%x\n", cpu_features);
+    }
 
   D();
   glade_gnome_init();
@@ -662,8 +656,8 @@ int main(int argc, char * argv[])
 	  fprintf (stderr, "Screen %d:\n"
 		   "  position               %u, %u - %u, %u\n"
 		   "  frame buffer address   0x%lx\n"
-		   "  frame buffer size      %ux%u pixels, 0x%x bytes\n"
-		   "  bytes per line         %u bytes\n"
+		   "  frame buffer size      %ux%u pixels, 0x%lx bytes\n"
+		   "  bytes per line         %lu bytes\n"
 		   "  pixfmt                 %s\n",
 		   xs->screen_number,
 		   xs->x,
@@ -686,12 +680,21 @@ int main(int argc, char * argv[])
   if (debug_msg)
     {
       x11_vidmode_info *v;
+      unsigned int i = 0;
+
+      fprintf (stderr, "VidModes:");
 
-      fprintf (stderr, "VidModes:\n");
       for (v = vidmodes; v; v = v->next)
-        fprintf (stderr, "  %ux%u@%u\n",
-		 v->width, v->height,
-		 (unsigned int)(v->vfreq + 0.5));
+	{
+	  fprintf (stderr, "%s%ux%u@%u",
+		   (0 == i) ? "\n  " : " ",
+		   v->width,
+		   v->height,
+		   (unsigned int)(v->vfreq + 0.5));
+	  i = (i + 1) % 5;
+	}
+
+      fputc ('\n', stderr);
     }
 
   /* Determine size and pixfmt of the Display. */
@@ -955,9 +958,6 @@ void shutdown_zapping(void)
   guint i = 0;
   gchar * buffer = NULL;
   tveng_tuned_channel * channel;
-  const tv_video_line *vi;
-  const tv_audio_line *ai;
-  const tv_video_standard *vs;
 
   printv("Shutting down the beast:\n");
 
@@ -1052,23 +1052,10 @@ void shutdown_zapping(void)
       i++;
     }
 
+  zconf_set_sources (zapping->info);
+
   tveng_tuned_channel_list_delete (&global_channel_list);
 
-  if ((vs = tv_cur_video_standard (zapping->info)))
-    zcs_uint (vs->hash, "current_standard");
-  else
-    zcs_int (0, "current_standard");
-
-  if ((vi = tv_cur_video_input (zapping->info)))
-    zcs_uint (vi->hash, "current_input");
-  else
-    zcs_int (0, "current_input");
-
-  if ((ai = tv_cur_audio_input (zapping->info)))
-    zcs_uint (ai->hash, "current_audio_input");
-  else
-    zcs_int (0, "current_audio_input");
-
   /* inputs, standards handling */
   printv("\n v4linterface");
   shutdown_v4linterface();
@@ -1125,7 +1112,7 @@ void shutdown_zapping(void)
    * The mixer code.
    */
   printv(" mixer");
-  shutdown_mixer();
+  shutdown_mixer(zapping->info);
 
   /*
    * The plugin properties dialog.
@@ -1304,16 +1291,18 @@ static gboolean startup_zapping(gboolean load_plugins,
   /* Loads the plugins */
   if (load_plugins)
     plugin_list = plugin_load_plugins();
-  D();
   /* init them, and remove the ones that couldn't be inited */
  restart_loop:
-
+  D();
   p = g_list_first(plugin_list);
   while (p)
     {
+      D();
       plugin_load_config((struct plugin_info*)p->data);
+      D();
       if (!plugin_init(info, (struct plugin_info*)p->data))
 	{
+	  D();
 	  plugin_unload((struct plugin_info*)p->data);
 	  plugin_list = g_list_remove_link(plugin_list, p);
 	  g_list_free_1(p);
