@@ -138,7 +138,6 @@ struct private_tveng25_device_info
 	struct v4l2_capability	caps;
 
 	struct xbuffer *	buffers;
-	unsigned int		n_buffers;
 
 	tv_control *		mute;
 
@@ -152,15 +151,15 @@ struct private_tveng25_device_info
 
 #define P_INFO(p) PARENT (p, struct private_tveng25_device_info, info)
 
-/* FIXME must be >= capture.c/N_BUNDLES. */
-#define N_BUFFERS 8
-
 #ifndef TVENG25_BAYER_TEST
 #define TVENG25_BAYER_TEST 0
 #endif
 
 static tv_bool
 queue_xbuffers			(tveng_device_info *	info);
+static tv_bool
+unmap_xbuffers			(tveng_device_info *	info,
+				 tv_bool		ignore_errors);
 
 /*
 	Audio matrix
@@ -618,7 +617,8 @@ add_control			(tveng_device_info *	info,
 		goto failure;
 	}
 
-	if (!(tc = append_control (info, &xc->pub, /* allocate size */ 0)))
+	if (!(tc = append_panel_control (info,
+					 &xc->pub, /* allocate size */ 0)))
 		goto failure;
 
 	get_xcontrol (info, C(tc));
@@ -630,7 +630,7 @@ add_control			(tveng_device_info *	info,
 	return TRUE;
 
  failure:
-	free_control (&xc->pub);
+	tv_control_delete (&xc->pub);
 
 	return TRUE; /* no control, but not end of enum */
 }
@@ -640,7 +640,7 @@ get_control_list		(tveng_device_info *	info)
 {
 	unsigned int cid;
 
-	free_controls (info);
+	free_panel_controls (info);
 
 	for (cid = V4L2_CID_BASE; cid < V4L2_CID_LASTP1; ++cid) {
 		/* EINVAL ignored */
@@ -840,6 +840,7 @@ set_tuner_frequency		(tveng_device_info *	info,
 	struct private_tveng25_device_info * p_info = P_INFO (info);
 	struct video_input *vi = VI (l);
 	struct v4l2_frequency vfreq;
+	unsigned long new_freq;
 	tv_bool restart;
 	int buf_type;
 	tv_bool r;
@@ -847,7 +848,8 @@ set_tuner_frequency		(tveng_device_info *	info,
 	CLEAR (vfreq);
 	vfreq.tuner = vi->tuner;
 	vfreq.type = V4L2_TUNER_ANALOG_TV;
-	vfreq.frequency = (frequency << vi->step_shift) / vi->pub.u.tuner.step;
+
+	new_freq = (frequency << vi->step_shift) / vi->pub.u.tuner.step;
 
 	buf_type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
@@ -856,12 +858,21 @@ set_tuner_frequency		(tveng_device_info *	info,
 		   && info->panel.cur_video_input == l);
 
 	if (restart) {
+		if (0 == xioctl (info, VIDIOC_G_FREQUENCY, &vfreq)) {
+			if (new_freq == vfreq.frequency) {
+				store_frequency (info, vi, new_freq);
+				return TRUE;
+			}
+		}
+
 		if (-1 == xioctl (info, VIDIOC_STREAMOFF, &buf_type))
 			return FALSE;
 
 		p_info->streaming = FALSE;
 	}
 
+	vfreq.frequency = new_freq;
+
 	r = TRUE;
 
 	if (0 == xioctl (info, VIDIOC_S_FREQUENCY, &vfreq))
@@ -874,7 +885,7 @@ set_tuner_frequency		(tveng_device_info *	info,
 
 		/* Some drivers do not completely fill buffers when
 		   tuning to an empty channel. */
-		for (i = 0; i < p_info->n_buffers; ++i)
+		for (i = 0; i < info->capture.n_buffers; ++i)
 			p_info->buffers[i].clear = TRUE;
 
 		if (!queue_xbuffers (info)) {
@@ -1242,11 +1253,15 @@ get_overlay_buffer		(tveng_device_info *	info)
 static tv_bool
 get_overlay_window		(tveng_device_info *	info)
 {
+	struct private_tveng25_device_info * p_info = P_INFO (info);
 	struct v4l2_format format;
 
 	CLEAR (format);
 	format.type = V4L2_BUF_TYPE_VIDEO_OVERLAY;
 
+	if (p_info->buffers)
+		unmap_xbuffers (info, /* ignore_errors */ TRUE);
+
 	if (-1 == xioctl (info, VIDIOC_G_FMT, &format))
 		return FALSE;
 
@@ -1314,6 +1329,9 @@ set_overlay_window_clipvec	(tveng_device_info *	info,
 
 	format.fmt.win.chromakey	= p_info->chroma;
 
+	if (p_info->buffers)
+		unmap_xbuffers (info, /* ignore_errors */ TRUE);
+
 	if (-1 == xioctl (info, VIDIOC_S_FMT, &format)) {
 		free (clips);
 		return FALSE;
@@ -1335,8 +1353,12 @@ static tv_bool
 enable_overlay			(tveng_device_info *	info,
 				 tv_bool		on)
 {
+	struct private_tveng25_device_info * p_info = P_INFO (info);
 	int value = on;
 
+	if (on && p_info->buffers)
+		unmap_xbuffers (info, /* ignore_errors */ TRUE);
+
 	if (0 == xioctl (info, VIDIOC_OVERLAY, &value)) {
 		usleep (50000);
 		return TRUE;
@@ -1482,6 +1504,9 @@ set_capture_format		(tveng_device_info *	info,
 	else
 		format.fmt.pix.field = V4L2_FIELD_BOTTOM;
 
+	if (p_info->buffers)
+		unmap_xbuffers (info, /* ignore_errors */ TRUE);
+
 	if (-1 == xioctl (info, VIDIOC_S_FMT, &format))
 		return FALSE;
 
@@ -1638,7 +1663,7 @@ queue_xbuffers			(tveng_device_info *	info)
 	struct private_tveng25_device_info *p_info = P_INFO (info);
 	unsigned int i;
 
-	for (i = 0; i < p_info->n_buffers; ++i) {
+	for (i = 0; i < info->capture.n_buffers; ++i) {
 		if (p_info->buffers[i].dequeued)
 			continue;
 
@@ -1660,7 +1685,7 @@ queue_buffer			(tveng_device_info *	info,
 	b = PARENT (buffer, struct xbuffer, cb);
 
 	index = b - p_info->buffers;
-	assert ((unsigned int) index < p_info->n_buffers);
+	assert ((unsigned int) index < info->capture.n_buffers);
 	assert (b == &p_info->buffers[index]);
 
 	assert (b->dequeued);
@@ -1819,7 +1844,7 @@ dequeue_buffer			(tveng_device_info *	info,
 		}
 	}
 
-	assert (vbuf.index < p_info->n_buffers);
+	assert (vbuf.index < info->capture.n_buffers);
 
 	b = p_info->buffers + vbuf.index; 
 
@@ -1857,7 +1882,7 @@ flush_buffers			(tveng_device_info *	info)
 		return FALSE;
 	}
 
-	for (count = p_info->n_buffers; count-- > 0;) {
+	for (count = info->capture.n_buffers; count-- > 0;) {
 		struct v4l2_buffer buffer;
 
 		CLEAR (buffer);
@@ -1934,7 +1959,7 @@ unmap_xbuffers			(tveng_device_info *	info,
 
 	success = TRUE;
 
-	for (i = p_info->n_buffers; i-- > 0;) {
+	for (i = info->capture.n_buffers; i-- > 0;) {
 		if (-1 == device_munmap (info->log_fp,
 					 p_info->buffers[i].cb.data,
 					 p_info->buffers[i].vb.length)) {
@@ -1950,12 +1975,12 @@ unmap_xbuffers			(tveng_device_info *	info,
 	}
 
 	memset (p_info->buffers, 0,
-		p_info->n_buffers * sizeof (* p_info->buffers));
+		info->capture.n_buffers * sizeof (* p_info->buffers));
 
 	free (p_info->buffers);
 
 	p_info->buffers = NULL;
-	p_info->n_buffers = 0;
+	info->capture.n_buffers = 0;
 
 	return success;
 }
@@ -1970,7 +1995,7 @@ map_xbuffers			(tveng_device_info *	info,
 	unsigned int i;
 
 	p_info->buffers = NULL;
-	p_info->n_buffers = 0;
+	info->capture.n_buffers = 0;
 
 	CLEAR (rb);
 
@@ -1985,7 +2010,7 @@ map_xbuffers			(tveng_device_info *	info,
 		return FALSE;
 
 	p_info->buffers = calloc (rb.count, sizeof (* p_info->buffers));
-	p_info->n_buffers = 0;
+	info->capture.n_buffers = 0;
 
 	for (i = 0; i < rb.count; ++i) {
 		struct xbuffer *b;
@@ -2023,7 +2048,7 @@ map_xbuffers			(tveng_device_info *	info,
 		b->dequeued = FALSE;
 		b->clear = TRUE;
 
-		++p_info->n_buffers;
+		++info->capture.n_buffers;
 	}
 
 	if (queue_xbuffers (info))
@@ -2033,7 +2058,7 @@ map_xbuffers			(tveng_device_info *	info,
 	info->tveng_errno = errno;
 	t_error("mmap()", info);
 
-	if (p_info->n_buffers > 0) {
+	if (info->capture.n_buffers > 0) {
 		/* Driver bug, out of memory? May still work. */
 
 		if (queue_xbuffers (info))
@@ -2045,6 +2070,56 @@ map_xbuffers			(tveng_device_info *	info,
 	return FALSE;
 }
 
+static tv_bool
+set_capture_buffers		(tveng_device_info *	info,
+				 tv_capture_buffer *	buffers,
+				 unsigned int		n_buffers)
+{
+	struct private_tveng25_device_info *p_info = P_INFO (info);
+
+	assert (NULL == buffers);
+
+	if (p_info->streaming
+	    || CAPTURE_MODE_NONE != info->capture_mode) {
+		info->tveng_errno = -1;
+		tv_error_msg (info, "Internal error (%s:%d)",
+			      __FUNCTION__, __LINE__);
+		return FALSE;
+	}
+
+	if (p_info->buffers)
+		unmap_xbuffers (info, /* ignore_errors */ TRUE);
+
+	if (0 == n_buffers) {
+		/* Mission accomplished. */
+		return TRUE;
+	}
+
+	assert (0 == info->capture.n_buffers);
+	assert (NULL == p_info->buffers);
+
+	/* sn9c102 1.0.8 bug: mmap fails for > 1 buffers, apparently
+	   because m.offset isn't page aligned. */
+	if (0 == strcmp (p_info->caps.driver, "sn9c102"))
+		n_buffers = 1;
+
+	if (!map_xbuffers (info,
+			   V4L2_BUF_TYPE_VIDEO_CAPTURE,
+			   n_buffers))
+		return FALSE;
+
+	if (info->capture.n_buffers < 1) {
+		unmap_xbuffers (info, /* ignore_errors */ TRUE);
+
+		info->tveng_errno = -1;
+		tv_error_msg(info, "Not enough buffers");
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static tv_bool
 enable_capture			(tveng_device_info *	info,
 				 tv_bool		enable)
@@ -2053,34 +2128,13 @@ enable_capture			(tveng_device_info *	info,
 
 	if (enable) {
 		gboolean dummy;
-		unsigned int n_buffers;
 		int buf_type;
 
 		p_tveng_stop_everything(info,&dummy);
 
-		assert (info -> capture_mode == CAPTURE_MODE_NONE);
-		assert (0 == p_info->n_buffers);
-		assert (NULL == p_info->buffers);
-
-		/* sn9c102 1.0.8 bug: mmap fails for > 1 buffers, apparently
-		   because m.offset isn't page aligned. */
-		if (0 == strcmp (p_info->caps.driver, "sn9c102"))
-			n_buffers = 1;
-		else
-			n_buffers = N_BUFFERS;
-
-		if (!map_xbuffers (info,
-				   V4L2_BUF_TYPE_VIDEO_CAPTURE,
-				   n_buffers))
-			return FALSE;
-
-		if (p_info->n_buffers < 1
-		    /* FIXME must be >= capture.c/N_BUNDLES. */
-		    || ((info->caps.flags & TVENG_CAPS_QUEUE)
-			&& p_info->n_buffers < n_buffers)) {
-			info->tveng_errno = -1;
-			tv_error_msg(info, "Not enough buffers");
-			return FALSE;
+		if (!p_info->buffers) {
+			if (!set_capture_buffers (info, NULL, 8))
+				return FALSE;
 		}
 
 		buf_type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -2121,10 +2175,14 @@ enable_capture			(tveng_device_info *	info,
 /* Closes a device opened with tveng_init_device */
 static void tveng25_close_device(tveng_device_info * info)
 {
+  struct private_tveng25_device_info *p_info = P_INFO (info);
   gboolean dummy;
  
   p_tveng_stop_everything(info,&dummy);
 
+  if (p_info->buffers)
+	  unmap_xbuffers (info, /* ignore_errors */ TRUE);
+
   device_close(info->log_fp, info->fd);
   info -> fd = -1;
   info -> current_controller = TVENG_CONTROLLER_NONE;
@@ -2135,7 +2193,7 @@ static void tveng25_close_device(tveng_device_info * info)
       info->file_name = NULL;
     }
 
-	free_controls (info);
+	free_panel_controls (info);
 	free_video_standards (info);
 	free_video_inputs (info);
 
@@ -2496,6 +2554,7 @@ int tveng25_attach_device(const char* device_file,
 	if (info->caps.flags & TVENG_CAPS_CAPTURE) {
 		info->capture.get_format = get_capture_format;
 		info->capture.set_format = set_capture_format;
+		info->capture.set_buffers = set_capture_buffers;
 		info->capture.read_frame = read_frame;
 		info->capture.queue_buffer = queue_buffer;
 	      	info->capture.dequeue_buffer = dequeue_buffer;
@@ -2510,7 +2569,7 @@ int tveng25_attach_device(const char* device_file,
 	}
 
   /* Init the private info struct */
-  p_info->n_buffers = 0;
+  info->capture.n_buffers = 0;
   p_info->buffers = NULL;
 
   return info -> fd;
