@@ -198,11 +198,19 @@ struct capture_device {
 	/* Preliminary. If zero try set_format. */
 	tv_pixfmt_set		supported_pixfmt_set;
 
+	unsigned int		n_buffers;
+
 	tv_bool			(* set_format)
 					(tveng_device_info *	info,
 					 const tv_image_format *format);
 	tv_bool			(* get_format)
 					(tveng_device_info *	info);
+
+	tv_bool			(* set_buffers)
+					(tveng_device_info *	info,
+					 tv_capture_buffer *	buffers,
+					 unsigned int		n_buffers);
+
 	/**
 	 * @param buffer The image will be stored here. Can be @c NULL
 	 *   to read and discard the image. buffer->format crops the image
@@ -324,14 +332,6 @@ struct _tveng_device_info
 
   pthread_mutex_t mutex; /* Thread safety */
 
-  /* Controls managed directly by tveng.c */
-#ifdef USE_XV
-  XvPortID	port;
-  Atom filter;
-  Atom double_buffer;
-  Atom colorkey; /* colorkey doesn't have min, max, it's defined by
-		    RGB triplets */
-#endif
 
   unsigned int		callback_recursion;
 
@@ -339,7 +339,7 @@ struct _tveng_device_info
   tv_control *		control_mute;
   tv_bool		quiet;
 
-
+  tv_control *		cloned_controls;
 };
 
 
@@ -397,9 +397,9 @@ store_cur_##item		(tveng_device_info *	info,		\
 
 NODE_HELPER_FUNCTIONS		(control, control);
 extern void
-free_controls			(tveng_device_info *	info);
+free_panel_controls		(tveng_device_info *	info);
 extern tv_control *
-append_control			(tveng_device_info *	info,
+append_panel_control		(tveng_device_info *	info,
 				 tv_control *		tc,
 				 unsigned int		size);
 extern tv_control *
