@@ -494,9 +494,7 @@ add_controls			(struct control_window *cb,
   cb->table = NULL;
   cb->index = 0;
 
-  ctrl = NULL;
-
-  while ((ctrl = tv_next_control (info, ctrl)))
+  for (ctrl = tv_next_control (info, NULL); ctrl; ctrl = ctrl->_next)
     {
       const tv_video_standard *vs;
 
@@ -660,9 +658,7 @@ update_control_box		(tveng_device_info *	info)
 
   c = ToolBox->controls;
 
-  ctrl = NULL;
-
-  while ((ctrl = tv_next_control (info, ctrl)))
+  for (ctrl = tv_next_control (info, NULL); ctrl; ctrl = ctrl->_next)
     {
       /* XXX Is this safe? Unlikely. */
       if (!c || c->ctrl != ctrl)
@@ -735,6 +731,9 @@ z_switch_video_input		(guint hash, tveng_device_info *info)
       return FALSE;
     }
 
+  if (l == tv_cur_video_input (info))
+    return TRUE;
+
   old_mode = tv_get_capture_mode (info);
   if (CAPTURE_MODE_READ == old_mode)
     {
@@ -748,8 +747,7 @@ z_switch_video_input		(guint hash, tveng_device_info *info)
   if (!tv_set_video_input (info, l))
     {
       if (CAPTURE_MODE_READ == old_mode) {
-	capture_start (info, GTK_WIDGET (zapping->video));
-	tveng_start_capturing (info);
+	capture_start (info, zapping->display_window);
       }
 
       ShowBox("Couldn't switch to video input %s\n%s",
@@ -759,8 +757,7 @@ z_switch_video_input		(guint hash, tveng_device_info *info)
     }
 
   if (CAPTURE_MODE_READ == old_mode) {
-    capture_start (info, GTK_WIDGET (zapping->video));
-    tveng_start_capturing (info);
+    capture_start (info, zapping->display_window);
   }
 
   zmodel_changed(z_input_model);
@@ -828,6 +825,9 @@ z_switch_standard		(guint hash, tveng_device_info *info)
     }
 #endif
 
+  if (s == tv_cur_video_standard (info))
+    return TRUE;
+
   old_mode = tv_get_capture_mode (info);
   if (CAPTURE_MODE_READ == old_mode)
     {
@@ -852,8 +852,7 @@ z_switch_standard		(guint hash, tveng_device_info *info)
   r = tv_set_video_standard (info, s);
 
   if (CAPTURE_MODE_READ == old_mode) {
-    capture_start (info, GTK_WIDGET (zapping->video));
-    tveng_start_capturing (info);
+    capture_start (info, zapping->display_window);
   }
 
 #ifdef HAVE_LIBZVBI
@@ -1006,9 +1005,7 @@ tveng_tc_control_by_id		(const tveng_device_info *info,
   const tv_control *c;
   guint i;
 
-  for (c = NULL; (c = tv_next_control (info, c));)
-    if (c->id == id)
-      break;
+  c = tv_control_by_id (info, id);
 
   if (NULL != c)
     for (i = 0; i < num_controls; ++i)
@@ -1032,9 +1029,7 @@ load_control_values		(tveng_device_info *	info,
   if (!tcc || num_controls == 0)
     return 0;
 
-  ctrl = NULL;
-
-  while ((ctrl = tv_next_control (info, ctrl)))
+  for (ctrl = tv_next_control (info, NULL); ctrl; ctrl = ctrl->_next)
     for (i = 0; i < num_controls; i++)
       if (normstrcmp (ctrl->label, tcc[i].name))
 	{
@@ -1080,9 +1075,7 @@ store_control_values		(tveng_device_info *	info,
   tcc = NULL;
   num_controls = 0;
 
-  ctrl = NULL;
-
-  while ((ctrl = tv_next_control (info, ctrl)))
+  for (ctrl = tv_next_control (info, NULL); ctrl; ctrl = ctrl->_next)
     num_controls++;
 
   if (num_controls > 0)
@@ -1115,6 +1108,70 @@ store_control_values		(tveng_device_info *	info,
   *num_controls_p = num_controls;
 }
 
+void
+zconf_get_sources		(tveng_device_info *	info,
+				 gboolean               mute)
+{
+  gint video_input;
+  gint audio_input;
+  gint video_standard;
+  tveng_tuned_channel *ch;
+
+  video_input = zcg_int (NULL, "current_input");
+  if (video_input)
+    z_switch_video_input (video_input, info);
+
+  audio_input = zcg_int (NULL, "current_audio_input");
+  if (audio_input)
+    z_switch_audio_input (audio_input, info);
+
+  video_standard = zcg_int (NULL, "current_standard");
+  if (video_standard)
+    z_switch_standard (video_standard, info);
+
+  cur_tuned_channel = zcg_int (NULL, "cur_tuned_channel");
+  ch = tveng_tuned_channel_nth (global_channel_list,
+				(guint) cur_tuned_channel);
+  if (NULL != ch)
+    {
+      if (mute)
+	{
+	  tveng_tc_control *mute;
+
+	  if ((mute = tveng_tc_control_by_id (info,
+					      ch->controls,
+					      ch->num_controls,
+					      TV_CONTROL_ID_MUTE)))
+	    mute->value = 1; /* XXX sub-optimal */
+	}
+
+      z_switch_channel (ch, info);
+    }
+}
+
+void
+zconf_set_sources		(tveng_device_info *	info)
+{
+  const tv_video_line *vl;
+  const tv_audio_line *al;
+  const tv_video_standard *vs;
+
+  vl = tv_cur_video_input (info);
+  zcs_uint (vl ? vl->hash : 0, "current_input");
+
+  al = tv_cur_audio_input (info);
+  zcs_uint (al ? al->hash : 0, "current_audio_input");
+
+  vs = tv_cur_video_standard (info);
+  zcs_uint (vs ? vs->hash : 0, "current_standard");
+
+  zcs_int (cur_tuned_channel, "cur_tuned_channel");
+}
+
+
+
+
+
 /*
   Substitute the special search keywords by the appropiate thing,
   returns a newly allocated string, and g_free's the given string.
@@ -1754,8 +1811,7 @@ append_radio_menu_item		(GtkMenuShell **	menu_shell,
 
   gtk_menu_shell_append (*menu_shell, menu_item);
 
-  z_signal_connect_const (G_OBJECT (menu_item), "activate",
-			  handler, sm);
+  z_signal_connect_const (G_OBJECT (menu_item), "toggled", handler, sm);
 }
 
 /* Video standards */
@@ -1826,6 +1882,9 @@ on_video_standard_activate	(GtkMenuItem *		menu_item,
   gboolean success;
   gint index;
 
+  if (!GTK_CHECK_MENU_ITEM (menu_item)->active)
+    return;
+
   menu_shell = GTK_MENU_SHELL (gtk_menu_item_get_submenu (sm->menu_item));
   if (!menu_shell)
     return;
@@ -1961,6 +2020,9 @@ on_audio_input_activate		(GtkMenuItem *		menu_item,
   gboolean success;
   gint index;
 
+  if (!GTK_CHECK_MENU_ITEM (menu_item)->active)
+    return;
+
   menu_shell = GTK_MENU_SHELL (gtk_menu_item_get_submenu (sm->menu_item));
   if (!menu_shell)
     return;
@@ -2098,6 +2160,9 @@ on_video_input_activate		(GtkMenuItem *		menu_item,
   gboolean success;
   gint index;
 
+  if (!GTK_CHECK_MENU_ITEM (menu_item)->active)
+    return;
+
   menu_shell = GTK_MENU_SHELL (gtk_menu_item_get_submenu (sm->menu_item));
   if (!menu_shell)
     return;
