@@ -30,13 +30,13 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#  include "config.h"
 #endif
 
 #include <gnome.h>
-#include <tree.h> /* libxml */
-#include <parser.h> /* libxml */
-#include <xmlmemory.h> /* libxml */
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -47,20 +47,7 @@
 #include "zconf.h"
 #include "zmisc.h" /* Misc common stuff */
 #include "zmodel.h"
-
-/*
-  Work around these little annoying incompatibilities between libxml 1
-  and 2
-*/
-#ifdef LIBXML_CHILDS
-#define children childs
-#endif
-
-#ifndef LIBXML_xmlDocGetRootElement
-/* Some people don't need this defines, but need the ones above */
-#define xmlDocSetRootElement(doc, node) (doc->root = node)
-#define xmlDocGetRootElement(doc) (doc->root)
-#endif
+#include "globals.h"
 
 /*
   This defines a key in the configuration tree.
@@ -186,7 +173,7 @@ gboolean
 zconf_init(const gchar * domain)
 {
   /* Hold here the home dir */
-  gchar * home_dir = g_get_home_dir();
+  const gchar * home_dir = g_get_home_dir();
   gchar * buffer = NULL; /* temporal storage */
   DIR * config_dir;
   xmlDocPtr doc;
@@ -203,7 +190,7 @@ zconf_init(const gchar * domain)
     {
       buffer = g_strconcat(domain,
 			   " cannot determine your home dir", NULL);
-      RunBox(buffer, GNOME_MESSAGE_BOX_ERROR);
+      RunBox("%s", GTK_MESSAGE_ERROR, buffer);
       g_free(buffer);
       return FALSE;
     }
@@ -217,7 +204,7 @@ zconf_init(const gchar * domain)
 		S_IXGRP | S_IROTH | S_IXOTH) == -1)
 	{
 	  RunBox("Cannot create config home dir, check your permissions",
-		  GNOME_MESSAGE_BOX_ERROR);
+		  GTK_MESSAGE_ERROR);
 	  g_free(buffer);
 	  return FALSE;
 	}
@@ -225,7 +212,7 @@ zconf_init(const gchar * domain)
       if (!config_dir)
 	{
 	  RunBox("Cannot open config home dir, this is weird",
-		  GNOME_MESSAGE_BOX_ERROR);
+		  GTK_MESSAGE_ERROR);
 	  g_free(buffer);
 	  return FALSE;
 	}
@@ -242,7 +229,11 @@ zconf_init(const gchar * domain)
   /* This is blocking, and will freeze the GUI, but shouldn't take
      long */
   zconf_file = g_strdup(buffer); 
-  doc = xmlParseFile(zconf_file);
+  doc = NULL;
+  if (g_file_test (zconf_file, G_FILE_TEST_EXISTS)) {
+    /* Spits out an error message if the file does not exist, hence. */
+    doc = xmlParseFile(zconf_file);
+  }
 
   if (!doc) /* There isn't any doc to parse */
     {
@@ -325,11 +316,11 @@ gboolean zconf_close(void)
   /* and we destroy the zconf tree */
   zconf_root = p_zconf_cut_branch(zconf_root);
 
-  if (xmlSaveFile(zconf_file, doc) == -1)
+  if (xmlSaveFormatFile(zconf_file, doc, TRUE) == -1)
     {
       ShowBox("Zapping cannot save configuration to disk\n"
 	      "You should have write permissions to your home dir...",
-	      GNOME_MESSAGE_BOX_ERROR);
+	      GTK_MESSAGE_ERROR);
       return FALSE; /* Error */
     }
 
@@ -348,7 +339,7 @@ gboolean zconf_close(void)
 /*
   Returns the last error zconf generated, 0 means the last operation
   was ok. It is useful for checking ambiguous return values:
-  if ((!zconf_get_integer(...)) && zconf_error())
+  if ((!zconf_get_int(...)) && zconf_error())
      Show_Error_Message();
   else
      Show_Success_Message();
@@ -364,7 +355,7 @@ gint zconf_error(void)
   is not NULL, the value is also stored in the location pointed to by
   where.
 */
-gint zconf_get_integer(gint * where, const gchar * path)
+gint zconf_get_int(gint * where, const gchar * path)
 {
   struct zconf_key * key;
   zconf_we = TRUE; /* Start with an error */
@@ -399,7 +390,7 @@ gint zconf_get_integer(gint * where, const gchar * path)
 /*
   Sets an integer value, creating the path to it if needed.
 */
-void zconf_set_integer(gint new_value, const gchar * path)
+void zconf_set_int(gint new_value, const gchar * path)
 {
   struct zconf_key * key;
   zconf_we = TRUE; /* Start with an error */
@@ -457,7 +448,7 @@ void zconf_set_integer(gint new_value, const gchar * path)
   value. Set it to NULL for leaving it undocumented (a empty string is
   considered as a documented value)
 */
-void zconf_create_integer(gint new_value, const gchar * desc,
+void zconf_create_int(gint new_value, const gchar * desc,
 			  const gchar * path)
 {
   struct zconf_key * key;
@@ -497,6 +488,22 @@ void zconf_create_integer(gint new_value, const gchar * desc,
   zconf_we = FALSE;
 }
 
+guint zconf_get_uint(guint * where, const gchar * path)
+{
+  return (guint) zconf_get_int ((gint *) where, path);
+}
+
+void zconf_set_uint(guint new_value, const gchar * path)
+{
+  zconf_set_int ((gint) new_value, path);
+}
+
+void zconf_create_uint(guint new_value, const gchar * desc,
+		       const gchar * path)
+{
+  zconf_create_int ((gint) new_value, desc, path);
+}
+
 /*
   Gets a string value. The returned string is statically allocated,
   will only be valid until the next call to zconf, so use g_strdup()
@@ -505,7 +512,7 @@ void zconf_create_integer(gint new_value, const gchar * desc,
   if where is not NULL, zconf will g_strdup the string itself, and
   place a pointer to it in where, that should be freed later with g_free.
 */
-gchar * zconf_get_string(gchar ** where, const gchar * path)
+const gchar * zconf_get_string(gchar ** where, const gchar * path)
 {
   struct zconf_key * key;
   zconf_we = TRUE; /* Start with an error */
@@ -989,7 +996,7 @@ gboolean zconf_set_description(const gchar * desc, const gchar * path)
   string, and it will only be valid until the next zconf call. if
   where is not NULL, a new copy will be g_strdup()'ed there.
 */
-gchar * zconf_get_nth(gint index, gchar ** where, const gchar * path)
+gchar * zconf_get_nth(guint index, gchar ** where, const gchar * path)
 {
   struct zconf_key * key;
   struct zconf_key * subkey; /* The found key */
@@ -1000,7 +1007,6 @@ gchar * zconf_get_nth(gint index, gchar ** where, const gchar * path)
   g_assert(*path != 0);
   g_assert(zconf_started == TRUE);
   g_assert(zconf_root != NULL);
-  g_assert(index >= 0);
 
   key = p_zconf_resolve(path, zconf_root);
 
@@ -1086,7 +1092,7 @@ enum zconf_type zconf_get_type(const gchar * path)
   allocated, it may be overwritten the next time you call any zconf
   function. Always succeeds (returns [Unknown] if the type is unknown :-)
 */
-char *
+const char *
 zconf_type_string(enum zconf_type type)
 {
   switch (type)
@@ -1206,8 +1212,8 @@ p_zconf_parse(xmlNodePtr node, xmlDocPtr doc, struct zconf_key ** skey,
     }
 
   /* Create and clear the struct */
-  new_key = g_malloc(sizeof(struct zconf_key));
-  memset(new_key, 0, sizeof(struct zconf_key));
+  new_key = g_malloc0(sizeof(*new_key));
+
   new_key -> parent = parent;
   new_key -> full_path = full_name;
   new_key -> name = g_strdup(name);
@@ -1282,7 +1288,7 @@ p_zconf_write(xmlNodePtr node, xmlDocPtr doc, struct zconf_key * key)
   GList * sub_key;
   xmlNodePtr new_node; /* The node we are about to add */
   gchar * str_value = NULL; /* Value in plain text */
-  gchar * str_type = NULL; /* String describing the type */
+  const gchar * str_type = NULL; /* String describing the type */
 
   g_assert(key != NULL);
   g_assert(doc != NULL);
@@ -1356,7 +1362,7 @@ p_zconf_cut_branch(struct zconf_key * key)
   g_free(key -> contents);
   g_free(key -> description);
   if (key->model)
-    gtk_object_destroy(GTK_OBJECT(key->model));
+    g_object_unref(G_OBJECT(key->model));
 
   for (p = g_list_first(key->hooked); p; p = p->next)
     {
@@ -1567,8 +1573,8 @@ p_zconf_create(const gchar * key, struct zconf_key * starting_dir)
     }
 
   /* The child didn't exist, create it */
-  sub_key = (struct zconf_key*) g_malloc(sizeof(struct zconf_key));
-  memset(sub_key, 0, sizeof(struct zconf_key));
+  sub_key = g_malloc0(sizeof(*sub_key));
+
   sub_key -> name = g_strdup(key_name_2);
   ptr = starting_dir -> full_path;
   g_assert(ptr != NULL);
@@ -1590,10 +1596,10 @@ p_zconf_create(const gchar * key, struct zconf_key * starting_dir)
 
 /* HOOKS HANDLING */
 static
-void on_key_model_changed		(GtkObject	*model,
+void on_key_model_changed		(GObject	*model,
 					 struct	zconf_hook *hook)
 {
-  struct zconf_key *key = gtk_object_get_user_data(model);
+  struct zconf_key *key = g_object_get_data(model, "key");
 
   g_assert(key != NULL);
   g_assert(hook != NULL);
@@ -1603,7 +1609,8 @@ void on_key_model_changed		(GtkObject	*model,
 }
 
 static struct zconf_hook*
-real_add_hook(const gchar * key_name, ZConfHook callback, gpointer data)
+real_add_hook(const gchar * key_name, ZConfHook callback, gpointer data,
+	      gboolean wakeup)
 {
   struct zconf_key *key;
   struct zconf_hook *hook;
@@ -1622,7 +1629,7 @@ real_add_hook(const gchar * key_name, ZConfHook callback, gpointer data)
   if (!key->model)
     {
       key->model = ZMODEL(zmodel_new());
-      gtk_object_set_user_data(GTK_OBJECT(key->model), key);
+      g_object_set_data(G_OBJECT(key->model), "key", key);
     }
 
   hook = g_malloc(sizeof(struct zconf_hook));
@@ -1633,9 +1640,12 @@ real_add_hook(const gchar * key_name, ZConfHook callback, gpointer data)
 
   key->hooked = g_list_append(key->hooked, hook);
 
-  gtk_signal_connect(GTK_OBJECT(key->model), "changed",
-		     GTK_SIGNAL_FUNC(on_key_model_changed),
-		     hook);
+  g_signal_connect(G_OBJECT(key->model), "changed",
+		   G_CALLBACK(on_key_model_changed),
+		   hook);
+
+  if (wakeup)
+    callback(key->full_path, key->contents, data);
 
   return hook;
 }
@@ -1643,11 +1653,11 @@ real_add_hook(const gchar * key_name, ZConfHook callback, gpointer data)
 void
 zconf_add_hook(const gchar * key_name, ZConfHook callback, gpointer data)
 {
-  real_add_hook(key_name, callback, data);
+  real_add_hook(key_name, callback, data, FALSE);
 }
 
 static void
-on_object_hook_destroyed	(GtkObject	*object,
+on_object_hook_destroyed	(GObject	*object,
 				 struct zconf_hook *hook)
 {
   struct zconf_key *key;
@@ -1665,20 +1675,21 @@ on_object_hook_destroyed	(GtkObject	*object,
 }
 
 void
-zconf_add_hook_while_alive(GtkObject *object,
+zconf_add_hook_while_alive(GObject *object,
 			   const gchar * key_name,
 			   ZConfHook callback,
-			   gpointer data)
+			   gpointer data,
+			   gboolean wakeup)
 {
   struct zconf_hook *hook =
-    real_add_hook(key_name, callback, data);
+    real_add_hook(key_name, callback, data, wakeup);
 
   if (!hook)
     return;
 
-  gtk_signal_connect(GTK_OBJECT(object), "destroy",
-		     GTK_SIGNAL_FUNC(on_object_hook_destroyed),
-		     hook);
+  g_signal_connect(object, "destroy",
+		   G_CALLBACK(on_object_hook_destroyed),
+		   hook);
 }
 
 void
@@ -1722,9 +1733,12 @@ zconf_remove_hook(const gchar * key_name, ZConfHook callback, gpointer data)
   g_free(hook->key);
   key->hooked = g_list_remove(key->hooked, hook);
 
-  gtk_signal_disconnect_by_func(GTK_OBJECT(key->model),
-				GTK_SIGNAL_FUNC(on_key_model_changed),
-				hook);
+  g_signal_handlers_disconnect_matched (key->model,
+					G_SIGNAL_MATCH_FUNC |
+					G_SIGNAL_MATCH_DATA,
+					0, 0, NULL,
+					on_key_model_changed,
+					hook);
 
   g_free(hook);
 }
@@ -1750,3 +1764,45 @@ zconf_touch(const gchar * key_name)
 
   zmodel_changed(key->model);
 }
+
+/*
+ *  Generic hooks
+ */
+
+void
+zconf_hook_widget_show		(const gchar *		key _unused_,
+				 gpointer		new_value_ptr,
+				 gpointer		user_data)
+{
+  gboolean show = * (gboolean *) new_value_ptr;
+  GtkWidget *widget = user_data;
+
+  if (show)
+    gtk_widget_show (widget);
+  else
+    gtk_widget_hide (widget);
+}
+
+void
+zconf_hook_toggle_button	(const gchar *		key _unused_,
+				 gpointer		new_value_ptr,
+				 gpointer		user_data)
+{
+  gboolean active = * (gboolean *) new_value_ptr;
+  GtkToggleButton *toggle_button = user_data;
+
+  if (active != gtk_toggle_button_get_active (toggle_button))
+    gtk_toggle_button_set_active (toggle_button, active);
+}
+
+void
+zconf_hook_check_menu		(const gchar *		key _unused_,
+				 gpointer		new_value_ptr,
+				 gpointer		user_data)
+{
+  gboolean active = * (gboolean *) new_value_ptr;
+  GtkCheckMenuItem *item = user_data;
+
+  if (active != item->active)
+    gtk_check_menu_item_set_active (item, active);
+}
